// This class is the manager of the overall planning process.

// 	Name: AccidentPlanner (class)
//	Author: Arjan Egges
//	Version: 1.0


class AccidentPlanner implements Planner {

   // The planning consists of five main stages:
   
   // Planning stage I		: normalize formal description
   // Planning stage II		: calculate the initial start and endpositions 
   // Planning stage III	: general route planning without accident modeling
   // Planning stage IV		: accident modeling
   // Planning stage V		: temporal planning
   
   // Error indicator
   private boolean errorsDetected;
      
   // private members
   private AccidentInfo ainfo;
   
   // Constructor
   public AccidentPlanner(AccidentInfo ai) {
      ainfo = ai;
      errorsDetected = false;
   }
   
   // Method that invokes the different planning stages
   public void plan() {
      // Check if there are any dynamic objects
      if (ainfo.getDynamicAmount() == 0) {
         Messenger.message("No dynamic objects detected.\n");
         return;
      }

      // Check consistency
      if (ainfo.getDynamicAmount() != ainfo.getDynInfoAmount()) {
         Messenger.message("Information is inconsistent. Could not plan accident.\n");
         errorsDetected = true;
         return;
      }

      // Check if all ids differ
      for (int i=0; i<ainfo.getDynamicAmount(); i++) {
         String currId = ainfo.getDynamicObject(i).getId();
         for (int j=i+1; j<ainfo.getDynamicAmount(); j++) {
            if (currId.equals(ainfo.getDynamicObject(j).getId())) {
               Messenger.message("Duplicate name detected: "+currId+"\n");
               errorsDetected = true;
               return;
            }
         }
      }
   
      // Planning stage I: normalize formal description
      Messenger.message("Planning stage I: (normalize formal description)...\n");
      PrePlanner pp0 = new PrePlanner(ainfo);
      startPlanner(pp0);
      if (errorsDetected)
         return;
         
      // Planning stage II: calculate the initial start and endpositions 
      Messenger.message("Planning stage II: (calculate the initial start and endpositions)...\n");
      PositionPlanner pp1 = new PositionPlanner(ainfo);
      startPlanner(pp1);
      if (errorsDetected)
         return;
      
      // Planning stage III: general route planning without accident modeling
      Messenger.message("Planning stage III: (general route planning)...\n");

      GeneralAccidentRoutePlanner garp = new GeneralAccidentRoutePlanner(ainfo);
      startPlanner(garp);
      if (errorsDetected)
         return;
      
      // Planning stage IV: accident modeling
      Messenger.message("Planning stage IV: (accident modeling)...\n");
      MultipleAccidentPlanner map = new MultipleAccidentPlanner(ainfo);
      startPlanner(map);
      if (errorsDetected)
         return;
      
      // Print the generated collisions
      for (int index=0; index < ainfo.getCollisionInfoAmount(); index++) {
         Messenger.message("Collision "+(index+1)+": [<");
         Messenger.message(ainfo.getDynamicObject(ainfo.getCollisionInfo(index).actorIndex).getId());
         Messenger.message("> - "+Constant.partToString(ainfo.getCollisionInfo(index).actorPart));
         Messenger.message(", <");
         if (ainfo.getCollisionInfo(index).victimKind == Constant.STATIC_OBJECT) {
            Messenger.message(ainfo.getStaticObject(ainfo.getCollisionInfo(index).victimIndex).getId());
            Messenger.message("> ] ");
         } else {
            Messenger.message(ainfo.getDynamicObject(ainfo.getCollisionInfo(index).victimIndex).getId());
            Messenger.message("> - "+Constant.partToString(ainfo.getCollisionInfo(index).victimPart));
            Messenger.message(" ] ");
         }
         Messenger.message("with coord. ("+ainfo.getCollisionInfo(index).collCoordX+", ");
         Messenger.message(ainfo.getCollisionInfo(index).collCoordY+")\n");
      }         
      
      // Planning stage V: temporal planning
      Messenger.message("Planning stage V: (temporal planning)...\n");
      TemporalPlanner tp = new TemporalPlanner(ainfo);
      startPlanner(tp);
      if (errorsDetected)
         return;
      
      // Print the trajectories
      for (int index=0; index<ainfo.getDynamicAmount(); index++) {
         System.out.println("Trajectory of <"+ainfo.getDynamicObject(index).getId()+">:");
         System.out.println(ainfo.getDynamicObject(index).getTrajectory());
      }
   }
   
   private void startPlanner(Planner p) {
      p.plan();
      if (p.errors())
         errorsDetected = true;
   }
   
   public boolean errors() {
      return errorsDetected;
   }
}