// This class parses the accident formalism and generates a simple accident universe that 
// contains all the objects recognised (without planning!)

// 	Name: FormalismParser (class)
//	Author: Arjan Egges
//	Version: 1.0

import javax.vecmath.*;


class FormalismParser {

   // Grammar:
   
   //   form 			--> staticPart dynamicPart accidentPart
   
   //   staticPart 		--> 'STATIC' leftBracket staticObject* rightBracket
   //	staticObject		--> roadObject | treeObject | trafficLightObject | levelCrossing | stopSign
   //   roadObject 		--> 'ROAD' leftbracket roadProperties rightBracket
   //   roadProperties 		--> 'KIND' equals ( 'crossroad' | 'straightroad' | 'turn_left' | 'turn_right' ) semicolon


   //	treeObject		--> 'TREE' leftBracket treeProperties rightBracket
   //	treeProperties		--> id coords

   //	trafficLightObject	--> 'LIGHT' leftBracket lightProperties rightBracket
   //	lightProperties		--> id colorType coords
   //	colorType		--> 'COLOR' equals ('red' | 'orange' | 'green' | 'inactive') semicolon
   
   //	levelCrossing		--> 'CROSSING' leftBracket coords rightBracket
   //	stopSign		--> 'STOPSIGN' leftBracket coords rightBracket
   
   //	dynamicPart 		--> 'DYNAMIC' leftbracket dynamicObject* rightBracket
   //	dynamicObject 		--> 'VEHICLE' leftBracket vehicleProperties rightBracket
   //	vehicleProperties 	--> id kind [initDir] [startsign] [endsign] [eventChain]
   //	id 			--> 'ID' equals <string> semicolon
   //	kind 			--> 'KIND' equals ('car' | 'truck') semicolon
   // 	initDir 		--> 'INITDIRECTION' equals ('north' | 'south'| 'east' | 'west') semicolon
   //	startsign 		--> 'STARTSIGN' equals <string> semicolon
   //	endsign 		--> 'ENDSIGN' equals <string> semicolonicalEvent)* ']'
   //	eventChain		--> 'CHAIN' leftBracket event* rightBracket
   //	event 			--> 'EVENT' leftBracket eventProperties rightBracket
   //	criticalEvent		--> 'CRITICAL_EVENT' leftBracket eventProperties rightBracket
   
   //	eventProperties 	--> eventKind
   //	eventKind 		--> 'KIND' equals ( 'driving_forward' | 'turn_left' | 'turn_right' 
   //					| 'stop' | 'depass' | 'change_lane_left' | 'change_lane_right') semicolon
   
   //	accidentPart		--> 'ACCIDENT' leftBracket collisionPart+ rightBracket
   //	collisionPart 		--> 'COLLISION' leftBracket collisionDescription rightBracket
   //	collisionDescription	--> actorDescription victimDescription [coords]
   //	actorDescription	--> 'ACTOR' equals <string> separator vehiclePart semicolon
   //	victimDescription	--> 'VICTIM' equals <string> separator vehiclePart semicolon
   //	vehiclePart		--> ('front' | 'rear' | 'leftside' | 'rightside' | 'unknown')
   //	coords			--> 'COORD' equals openBracket <number> separator <number> closeBracket semicolon
   
   //	equals			--> "="
   //	semicolon		--> ";"
   //	separator		--> ","
   //	leftBracket		--> "["
   //	rightBracket		--> "]"
   //	openBracket		--> "("
   //	closeBracket		--> ")"
   
   
   private StringParser sParser;
   private AccidentUniverse au;
   private int criticalCount;
   private int errorCount;
   
   // Variables used by the Accident Planner
   AccidentInfo ainfo;
   
   // Constants
   private static final char leftBracket = '[';
   private static final char rightBracket = ']';
   private static final char openBracket = '(';
   private static final char closeBracket = ')';
   private static final char equalsSign = '=';
   private static final char semicolon = ';';
   private static final char separator = ',';
   
   public FormalismParser() {
      // Initialise the accidentUniverse
      au = new AccidentUniverse();
      errorCount = 0;
      criticalCount = 0;
   }
   
   public void parse(String ps) {
      sParser = new StringParser(ps);
      ainfo = au.getAccidentInfo();
      
      Messenger.message("Parsing...\n");
      form();
      Messenger.message(String.valueOf(errorCount)+" error(s) found.\n");
   }
   
   // Parser
   
   private void form() {
      staticPart();
      dynamicPart();
      accidentPart();
      if (!sParser.endOfString()) {
         String s = "";
         while (!sParser.endOfString()) 
            s = s + sParser.nextToken();
         errorCount++;
         Messenger.message(String.valueOf(sParser.currLine())+": Unexpected: "+s+"\n");
      }
      Messenger.message("Number of static objects: "+String.valueOf(ainfo.getStaticAmount())+"\n");
      Messenger.message("Number of dynamic objects: "+String.valueOf(ainfo.getDynamicAmount())+"\n");
   }
   
   private void staticPart() {
      if (sParser.endOfString())
         return;   
      read("STATIC");
      read(leftBracket);
      while (sParser.currChar() != rightBracket && !sParser.endOfString())
         staticObject();
      read(rightBracket);
   }
   
   private void staticObject() {
      if (sParser.currChar() == 'R') 
         roadObject();
      else if (sParser.currChar() == 'T')
         treeObject();
      else if (sParser.currChar() == 'L')
         trafficLightObject();
      else if (sParser.currChar() == 'C')
         levelCrossing();
      else if (sParser.currChar() == 'S')
         stopSign();
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: static object\n");
         errorCount++;
      }      
   }

   private void levelCrossing() {
      read("CROSSING");
      read(leftBracket);
      LevelCrossing lc = new LevelCrossing();
      Point2f p = readCoords();
      lc.xPos = p.x;
      lc.yPos = p.y;
      read(rightBracket);
      au.add(lc);
   }
   
   
   private void stopSign() {
      read("STOPSIGN");
      read(leftBracket);
      StopSign ss = new StopSign();
      Point2f p = readCoords();
      ss.xPos = p.x;
      ss.yPos = p.y;
      read(rightBracket);
      au.add(ss);
   }
   
   
   private void treeObject() {
      read("TREE");
      read(leftBracket);
      treeProperties();
      read(rightBracket);
   }
   
   private void treeProperties() {
      Tree t = new Tree();
      t.id = id();
      Point2f p = readCoords();
      t.xPos = p.x;
      t.yPos = p.y;
      au.add(t);    
   }
   

   private void roadObject() {
      read("ROAD");
      read(leftBracket);
      roadProperties();
      read(rightBracket);
   }
   
   private void roadProperties() {
      read("KIND");     
      read(equalsSign);         
      String keyW = sParser.nextToken();
      if (keyW.equals("crossroad")) {
         // Create a crossroad and add it to the universe
         Road crossroad = new Road();
         crossroad.type = Constant.CROSSROAD;
         au.add(crossroad);
      } else if (keyW.equals("straightroad")) {
         // Create a straightroad and add it to the universe
         Road straightroad = new Road();
         au.add(straightroad);
      } else if (keyW.equals("turn_left")) {
         // Create a left turn and add it to the universe
         Road turnleft = new Road();
         turnleft.type = Constant.TURNLEFT;
         au.add(turnleft);
      } else if (keyW.equals("turn_right")) {
         // Create a right turn and add it to the universe
         Road turnright = new Road();
         turnright.type = Constant.TURNRIGHT;
         au.add(turnright);        
      } else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: roadtype\n");
         errorCount++;
      }   
      read(semicolon);
   }
   
   private void trafficLightObject() {
      read("LIGHT");
      read(leftBracket);
      lightProperties();
      read(rightBracket);
   }
   
   private void lightProperties() {
      TrafficLight rtl = new TrafficLight();
      rtl.id = id();
      colorType(rtl);
      Point2f p = readCoords();
      rtl.xPos = p.x;
      rtl.yPos = p.y;
      au.add(rtl);    
   }
   
   private void colorType(TrafficLight rtl) {
      read("COLOR");      
      read(equalsSign);         
      String keyW = sParser.nextToken();
      if (keyW.equals("red")) 
         rtl.color = Constant.RED;
      else if (keyW.equals("orange")) 
         rtl.color = Constant.ORANGE;
      else if (keyW.equals("green")) 
         rtl.color = Constant.GREEN;
      else if (keyW.equals("inactive")) 
         rtl.color = Constant.INACTIVE;      
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: colorType\n");
         errorCount++;
      }   
      read(semicolon);
   }
   
   private void dynamicPart() {
      if (sParser.endOfString())
         return;
      read("DYNAMIC");
      read(leftBracket);
      while (sParser.currChar() != rightBracket && !sParser.endOfString())
         dynamicObject();
      read(rightBracket);
   }
   
   private void dynamicObject() {
      read("VEHICLE");
      read(leftBracket);
      vehicleProperties();
      read(rightBracket);
   }
   
   private void vehicleProperties() {
      DynamicAccidentObject obj;
      DynamicInfo currDynInfo = new DynamicInfo();
      String i = id();
      obj = kind(currDynInfo);
      obj.setId(i);
      while (!sParser.endOfString() && sParser.currChar() != rightBracket) {
         initDir(currDynInfo);
         startsign(currDynInfo);
         endsign(currDynInfo);
         eventChain(currDynInfo);
      }
      ainfo.addDynamicInfo(currDynInfo);
      au.add(obj);    
   }
      
   private DynamicAccidentObject kind(DynamicInfo currDynInfo) {
      DynamicAccidentObject d;
      SimpleCar s = new SimpleCar();
      d = s;
      read("KIND");     
      read(equalsSign);         
      String keyW = sParser.nextToken();
      if (keyW.equals("car")) {
         // Create a 2CV
         FrenchDuck fDuck = new FrenchDuck();
         d = fDuck;
      } else if (keyW.equals("truck")) {
         // Create a truck
         SimpleCar vehicle = new SimpleCar();
         vehicle.length = 6.0f;
         vehicle.height = 2.0f;
         d = vehicle;
      } else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: vehicletype\n");
         errorCount++;
      }   
      read(semicolon);
      return d;
   }
   
   private String id() {
      read("ID");      
      read(equalsSign);  
      // Read the id string.
      String keyW = "";
      while (sParser.currChar() != ';' && !sParser.endOfString()) {
         if (keyW.equals(""))
            keyW = keyW + sParser.nextToken();
         else
            keyW = keyW + " " + sParser.nextToken();
      } 
      String retId = keyW;
      read(semicolon);
      return retId;
   }
   
   private void initDir(DynamicInfo currDynInfo) {
      if (sParser.currChar() != 'I')
         return;
      read("INITDIRECTION");     
      read(equalsSign);  
      // Read the initdirection.       
      String keyW = sParser.nextToken();
      if (keyW.equals("north")) 
         currDynInfo.initDirection = DynamicInfo.NORTH;
      else if (keyW.equals("south")) 
         currDynInfo.initDirection = DynamicInfo.SOUTH;
      else if (keyW.equals("east")) 
         currDynInfo.initDirection = DynamicInfo.EAST;
      else if (keyW.equals("west")) 
         currDynInfo.initDirection = DynamicInfo.WEST;
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: directiontype\n");
         errorCount++;
      }
      read(semicolon);
   }
   
   private void startsign(DynamicInfo currDynInfo) {
      if (sParser.currChar() != 'S')
         return;
      read("STARTSIGN");     
      read(equalsSign); 
      // Read the startsign string.       
      String keyW = "";
      while (sParser.currChar() != ';' && !sParser.endOfString()) {
         keyW = keyW + sParser.nextToken() + " ";
      } 
      // Create a roadSign.
      Roadsign rs = new Roadsign(keyW);
      rs.xPos = -10.0f;
      au.add(rs);
      read(semicolon);
      currDynInfo.startsignIndex = ainfo.getStaticAmount()-1;

   }
   
   private void endsign(DynamicInfo currDynInfo) {
      if (sParser.currChar() != 'E')
         return;
      read("ENDSIGN");      
      read(equalsSign); 
      // Read the endsign string.       
      String keyW = "";
      while (sParser.currChar() != ';' && !sParser.endOfString()) {
         keyW = keyW + sParser.nextToken() + " ";
      } 
      // Create a roadSign.
      Roadsign rs = new Roadsign(keyW);
      rs.xPos = 10.0f;
      au.add(rs);
      read(semicolon);
      currDynInfo.endsignIndex = ainfo.getStaticAmount()-1;

   }
   
   private void eventChain(DynamicInfo currDynInfo) {
      criticalCount = 0;
      if (sParser.currChar() != 'C')
         return;
      currDynInfo.ceb = new EventChain();
      read("CHAIN");
      read(leftBracket);
      while (sParser.currChar() != rightBracket && !sParser.endOfString()) {
         event(currDynInfo);
         criticalEvent(currDynInfo);
      }
      read(rightBracket);
      if (criticalCount > 1) {
         Messenger.message(String.valueOf(sParser.currLine())+": Only 1 critical event allowed in an event chain\n");
         errorCount++;
      }
      
   }

   private void event(DynamicInfo currDynInfo) {
      if (sParser.currChar() != 'E')
         return;
      Event e = new Event();
      read("EVENT");
      read(leftBracket);
      eventKind(e);
      currDynInfo.ceb.addEvent(e);
      read(rightBracket);
   }

   private void criticalEvent(DynamicInfo currDynInfo) {
      if (sParser.currChar() != 'C')
         return;
      Event e = new Event();
      e.critical = true;
      read("CRITICAL_EVENT");
      read(leftBracket);
      eventKind(e);
      currDynInfo.ceb.addEvent(e);
      read(rightBracket);
      criticalCount++;
   }

   
   private void eventKind(Event e) {
      read("KIND");      
      read(equalsSign);         
      String keyW = sParser.nextToken();
      if (keyW.equals("driving_forward"))
         e.kind = Constant.STRAIGHTROAD;
      else if (keyW.equals("turn_left"))
         e.kind = Constant.TURNLEFT;
      else if (keyW.equals("turn_right"))
         e.kind = Constant.TURNRIGHT;
      else if (keyW.equals("stop")) 
         e.kind = Constant.STOP;
      else if (keyW.equals("depass"))
         e.kind = Constant.DEPASS;
      else if (keyW.equals("change_lane_left"))
         e.kind = Constant.CHANGELANE_LEFT;
      else if (keyW.equals("change_lane_right"))
         e.kind = Constant.CHANGELANE_RIGHT;
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: eventtype\n");
         errorCount++;
      }   
      read(semicolon);
   }
   
   private void accidentPart() {
      if (sParser.endOfString())
         return; 
      read("ACCIDENT");
      read(leftBracket);
      collisionPart();
      while (sParser.currChar() != rightBracket && !sParser.endOfString())
         collisionPart();
      read(rightBracket);
   }

   
   private void collisionPart() {
      if (sParser.endOfString())
         return;   
      read("COLLISION");
      read(leftBracket);
      collisionDescription();
      read(rightBracket);
   }
   
   private void collisionDescription() {
      CollisionInfo ci = new CollisionInfo();
      actorDescription(ci);
      victimDescription(ci);
      Point2f p = readCoords();
      ci.collCoordX = p.x;
      ci.collCoordY = p.y;
      ainfo.addCollisionInfo(ci);
   }
   
   
   private void actorDescription(CollisionInfo ci) {
      read("ACTOR");     
      read(equalsSign);     
      // Read the id-string   
      String keyW = "";
      while (sParser.currChar() != ',' && !sParser.endOfString()) {
         if (keyW.equals(""))
            keyW = sParser.nextToken();
         else
            keyW = keyW + " " + sParser.nextToken();
      } 
      // Search the corresponding index
      boolean found = false;
      int i=0;
      while (!found && i<ainfo.getDynamicAmount()) {
         if (keyW.equals(ainfo.getDynamicObject(i).getId()))
            found = true;
         else
            i++;
      }
      if (found) 
         ci.actorIndex = i;
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": vehiclename <"+keyW+"> not found.\n");
         errorCount++;
      }
      read(separator);     
      ci.actorPart = vehiclePart(); 
      read(semicolon);
   }
   
   private int vehiclePart() {
      String keyW = sParser.nextToken();
      int part = 0;
      if (keyW.equals("front")) {
         part = Constant.FRONT;
      } else if (keyW.equals("rear")) {
         part = Constant.REAR;
      } else if (keyW.equals("leftside")) {
         part = Constant.LEFTSIDE;
      } else if (keyW.equals("rightside")) {
         part = Constant.RIGHTSIDE;
      } else if (keyW.equals("unknown")) {
         part = Constant.UNKNOWN_PART;
      } else {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: vehiclepart\n");
         errorCount++;
      } 
      return part;  
   }
   
   private void victimDescription(CollisionInfo ci) {
      read("VICTIM");      
      read(equalsSign);      
      // Read the id-string   
      String keyW = "";
      while (sParser.currChar() != ',' && !sParser.endOfString()) {
         if (keyW.equals(""))
            keyW = sParser.nextToken();
         else
            keyW = keyW + " " + sParser.nextToken();
      } 
      // Search the corresponding index
      boolean found = false;
      int i=0;
      while (!found && i<ainfo.getDynamicAmount()) {
         if (keyW.equals(ainfo.getDynamicObject(i).getId()))
            found = true;
         else
            i++;
      }
      // The victim can be a static object as well
      if (!found) {
         i = 0;
         while (!found && i<ainfo.getStaticAmount()) {
            if (keyW.equals(ainfo.getStaticObject(i).getId())) {
               found = true;
               ci.victimKind = Constant.STATIC_OBJECT;
            } else
               i++;
         }
      }
         
      if (found) 
         ci.victimIndex = i;
      else {
         Messenger.message(String.valueOf(sParser.currLine())+": vehiclename <"+keyW+"> not found.\n");
         errorCount++;
      }
      read(separator);     
      ci.victimPart = vehiclePart();
      read(semicolon);
   }
   
   // Private methods for reading general strings and chars
   
   private void read(char c) {
      String keyW = sParser.nextToken();
      if (!keyW.equals(String.valueOf(c))) {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: "+String.valueOf(c)+"\n");
         errorCount++;
      }
   }
   
   private void read(String s) {
      String keyW = sParser.nextToken();
      if (!keyW.equals(s)) {
         Messenger.message(String.valueOf(sParser.currLine())+": Expected: "+s+"\n");
         errorCount++;
      }
   }

   private Point2f readCoords() {
      Point2f p = new Point2f(0.0f,0.0f);
      if (sParser.currChar() != 'C')
         return p;
      read("COORD");     
      read(equalsSign);  
      read(openBracket);  
      // Read the first number
      String keyW = sParser.nextToken();
      try {
         p.x = Float.parseFloat(keyW);
      } catch (Exception e) {
         errorCount++;
         Messenger.message(String.valueOf(sParser.currLine())+": <"+keyW+"> is not a valid x-coordinate.\n");
      }
      read(separator);  
      // Read the second number
      keyW = sParser.nextToken();
      try {
         p.y = Float.parseFloat(keyW);
      } catch (Exception e) {
         errorCount++;
         Messenger.message(String.valueOf(sParser.currLine())+": <"+keyW+"> is not a valid y-coordinate.\n");
      }
      read(closeBracket); 
      read(semicolon);
      return p;
   }
   
   public AccidentUniverse getUniverse() {
      return au;
   }
   
   public boolean errors() {
      return (errorCount>0);
   }
}