// This class provides methods for general route planning of a trafic situation
// on a crossroad, straightroad or a turn.

// 	Name: GeneralAccidentRoutePlanner (class)
//	Author: Arjan Egges
//	Version: 1.0


class GeneralAccidentRoutePlanner implements Planner {

   // scenarios:
   
   // I.	[driving_forward'], turn_left, [driving_forward']
   // II.	[driving_forward'], turn_right, [driving_forward']
   // III.	[driving_forward'], depass, [driving_forward']
   // IV.	[driving_forward'], changelane_left, [driving_forward']
   // V.	[driving_forward'], changelane_right, [driving_forward']
   // VI.	driving_forward'
   
   // driving_forward' = (driving_forward || stop)+
   
   
   
   // Planning properties
   public float startPoint = 20.0f;
   public float endPoint = 20.0f;
   public float depassStartPoint = 25.0f;
   public float depassEndPoint = 25.0f;
   public float depassStart = 14.0f;
   
   // private members
   private AccidentInfo ainfo;
   private int currScenario;
   private boolean errorsDetected = false;
   
   // internal variables used by the planner
   private int drivingforwardCount = 0;
   private int stopCount = 0;

   
   public GeneralAccidentRoutePlanner(AccidentInfo ai) {
      ainfo = ai;
      currScenario = Constant.UNKNOWN_SCENARIO;
   }
   
   public void plan() {
      
      for (int i=0; i<ainfo.getDynamicAmount(); i++) {
         drivingforwardCount = 0;
         stopCount = 0;
         scenario(i);
         if (currScenario == Constant.UNKNOWN_SCENARIO) {
            errorsDetected = true;
            return;
         }
         createEventChain(i);
         // Plan the trajectory and add it to the vehicle
         TrajectoryPlanner tp = new TrajectoryPlanner(ainfo.getDynamicInfo(i).ceb);
         tp.plan();
         ainfo.getDynamicObject(i).setTrajectory(tp.getTrajectory());
      }
   }
   
   // This method tries to determine the right scenario that corresponds to the dynamic object.
   
   public void scenario(int index) {
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      int i = 0;
      currScenario = Constant.UNKNOWN_SCENARIO;
      for (i = 0; i<ceb.getEventAmount(); i++) {
         Event currEvent = ceb.getEvent(i);
         // Set the right name
         currEvent.name = "event_"+String.valueOf(i+1)+": "+ainfo.getDynamicObject(index).getId()
                            +Constant.kindToString(currEvent.kind);
         if (currEvent.kind == Constant.STRAIGHTROAD) {
            drivingforwardCount++;
            if (currScenario == Constant.UNKNOWN_SCENARIO)
               currScenario = Constant.SCENARIO_VI;
         } else if (currEvent.kind == Constant.STOP) {
            stopCount++;
            if (currScenario == Constant.UNKNOWN_SCENARIO)
               currScenario = Constant.SCENARIO_VI;
         } else if (currEvent.kind == Constant.TURNLEFT) {
            if (currScenario == Constant.UNKNOWN_SCENARIO || currScenario == Constant.SCENARIO_VI)
               currScenario = Constant.SCENARIO_I;
            else {
               currScenario = Constant.UNKNOWN_SCENARIO;
               return;
            }
         } else if (currEvent.kind == Constant.TURNRIGHT) {
            if (currScenario == Constant.UNKNOWN_SCENARIO || currScenario == Constant.SCENARIO_VI)
               currScenario = Constant.SCENARIO_II;
            else {
               currScenario = Constant.UNKNOWN_SCENARIO;
               return;
            }
         } else if (currEvent.kind == Constant.DEPASS) {
            if (currScenario == Constant.UNKNOWN_SCENARIO || currScenario == Constant.SCENARIO_VI)
               currScenario = Constant.SCENARIO_III;
            else {
               currScenario = Constant.UNKNOWN_SCENARIO;
               return;
            }
         } else if (currEvent.kind == Constant.CHANGELANE_LEFT) {
            if (currScenario == Constant.UNKNOWN_SCENARIO || currScenario == Constant.SCENARIO_VI)
               currScenario = Constant.SCENARIO_IV;
            else {
               currScenario = Constant.UNKNOWN_SCENARIO;
               return;
            }
         } else if (currEvent.kind == Constant.CHANGELANE_RIGHT) {
            if (currScenario == Constant.UNKNOWN_SCENARIO || currScenario == Constant.SCENARIO_VI)
               currScenario = Constant.SCENARIO_V;
            else {
               currScenario = Constant.UNKNOWN_SCENARIO;
               return;
            }
         } else {
            currScenario = Constant.UNKNOWN_SCENARIO;
            return;
         }
      }     
   }

   // This method calls the method corresponding to the scenario that is determined.
   public void createEventChain(int index) {
      // At this stage the scenario is determined.
      Messenger.message("Scenario detected: "+ainfo.getDynamicObject(index).getId()+", "+Constant.scenarioToString(currScenario)+"\n");
      
      // Set the right start and endpoint
      startPoint = ainfo.getDynamicInfo(index).startPoint;
      endPoint = ainfo.getDynamicInfo(index).endPoint;
            
      // Choose the right scenario
      if (currScenario == Constant.SCENARIO_I)
         scenarioI(index);
      else if (currScenario == Constant.SCENARIO_II)
         scenarioII(index);
      else if (currScenario == Constant.SCENARIO_III)
         scenarioIII(index);
      else if (currScenario == Constant.SCENARIO_IV)
         scenarioIV(index);
      else if (currScenario == Constant.SCENARIO_V)
         scenarioV(index);
      else if (currScenario == Constant.SCENARIO_VI)
         scenarioVI(index);
      else {
         errorsDetected = true;
         return;
      }
   }
   
   // This method generates the event chain of SCENARIO_I
   private void scenarioI(int index) {
      
      // Count the number of driving_forward & stop units before:
      int eventNo = 0;
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      while (ceb.getEvent(eventNo).kind != Constant.TURNLEFT) {
         eventNo++;
      }
      
      // Count the number of driving_forward & stop units after:
      int eventNoAfter = ceb.getEventAmount() - eventNo - 1;
      
      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosX = -startPoint;
         startEvent.startPosY = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = -startPoint + 14.0f;
         endEvent.endPosY = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the left turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = 1.5f;
         e2.startPosX = -startPoint + 14.0f;
         e2.endPosY = -6.0f;
         e2.endPosX = 1.5f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = 1.5f;
            startEvent.startPosY = -6.0f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = 1.5f;
            endEvent.endPosY = -endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosX = startPoint;
         startEvent.startPosY = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = startPoint - 14.0f;
         endEvent.endPosY = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the left turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = -1.5f;
         e2.startPosX = startPoint - 14.0f;
         e2.endPosY = 6.0f;
         e2.endPosX = -1.5f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = -1.5f;
            startEvent.startPosY = 6.0f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = -1.5f;
            endEvent.endPosY = endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = startPoint;
         startEvent.startPosX = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = 1.5f;
         endEvent.endPosY = startPoint - 14.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the left turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = startPoint - 14.0f;
         e2.startPosX = 1.5f;
         e2.endPosY = -1.5f;
         e2.endPosX = -6.0f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = -6.0f;
            startEvent.startPosY = -1.5f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = -1.5f;
            endEvent.endPosX = -endPoint;
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -startPoint;
         startEvent.startPosX = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = -1.5f;
         endEvent.endPosY = -startPoint + 14.0f;
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the left turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = -startPoint + 14.0f;
         e2.startPosX = -1.5f;
         e2.endPosY = 1.5f;
         e2.endPosX = 6.0f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = 6.0f;
            startEvent.startPosY = 1.5f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = 1.5f;
            endEvent.endPosX = endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } 
        
   }

   // This method generates the event chain of SCENARIO_II
   private void scenarioII(int index) { 
      // Count the number of driving_forward & stop units before:
      int eventNo = 0;
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      while (ceb.getEvent(eventNo).kind != Constant.TURNRIGHT) {
         eventNo++;
      }
      
      // Count the number of driving_forward & stop units after:
      int eventNoAfter = ceb.getEventAmount() - eventNo - 1;
      
      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosX = -startPoint;
         startEvent.startPosY = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = -startPoint + 14.0f;
         endEvent.endPosY = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the right turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = 1.5f;
         e2.startPosX = -startPoint + 14.0f;
         e2.endPosY = 6.0f;
         e2.endPosX = -1.5f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = -1.5f;
            startEvent.startPosY = 6.0f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = -1.5f;
            endEvent.endPosY = endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosX = startPoint;
         startEvent.startPosY = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = startPoint - 14.0f;
         endEvent.endPosY = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the right turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = -1.5f;
         e2.startPosX = startPoint - 14.0f;
         e2.endPosY = -6.0f;
         e2.endPosX = 1.5f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = 1.5f;
            startEvent.startPosY = -6.0f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = 1.5f;
            endEvent.endPosY = -endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = startPoint;
         startEvent.startPosX = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = 1.5f;
         endEvent.endPosY = startPoint - 14.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the right turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = startPoint - 14.0f;
         e2.startPosX = 1.5f;
         e2.endPosY = 1.5f;
         e2.endPosX = 6.0f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = 6.0f;
            startEvent.startPosY = 1.5f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = 1.5f;
            endEvent.endPosX = endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -startPoint;
         startEvent.startPosX = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosX = -1.5f;
         endEvent.endPosY = -startPoint + 14.0f;
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();
         
         // Set the right turn properties
         Event e2 = ceb.getEvent(eventNo);
         e2.startPosY = -startPoint + 14.0f;
         e2.startPosX = -1.5f;
         e2.endPosY = -1.5f;
         e2.endPosX = -6.0f;
            
         // Set the right begin and end coordinates
         if (eventNoAfter > 0) {
            startEvent = ceb.getEvent(eventNo+1);
            startEvent.startPosX = -6.0f;
            startEvent.startPosY = -1.5f;
            endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = -1.5f;
            endEvent.endPosX = -endPoint;
         
            // Make a pathplanner and start it
            PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
            ppAfter.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      }        
   }

   // This method generates the event chain of SCENARIO_III
   private void scenarioIII(int index) {
      
      // Count the number of driving_forward & stop units before:
      int eventNo = 0;
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      while (ceb.getEvent(eventNo).kind != Constant.DEPASS) {
         eventNo++;
      }
      
      // Count the number of driving_forward & stop units after:
      int eventNoAfter = ceb.getEventAmount() - eventNo - 1;
      
      EventChain oldChain = ainfo.getDynamicInfo(index).ceb;
      EventChain newChain = new EventChain();
      // Copy all the events + three new ones into the new chain
      for (int j=0; j<eventNo; j++)
         newChain.addEvent(oldChain.getEvent(j));
      
      // Set some values in the eventchain
      Event changelaneLeft = new Event();	// Changelane left event
      changelaneLeft.name = "event_XXX (generated by planner): "+ainfo.getDynamicObject(index).getId()+" changing lane/toleft";
      changelaneLeft.kind = Constant.CHANGELANE_LEFT;
      newChain.addEvent(changelaneLeft);
      
      Event drivingForward = new Event();	// Depassing event
      drivingForward.name = "event_XXX (generated by planner): "+ainfo.getDynamicObject(index).getId()+" driving forward";
      drivingForward.kind = Constant.STRAIGHTROAD;
      newChain.addEvent(drivingForward);
      
      Event changelaneRight = new Event();	// Changelane right event
      changelaneRight.name = "event_XXX (generated by planner): "+ainfo.getDynamicObject(index).getId()+" changing lane/toright";
      changelaneRight.kind = Constant.CHANGELANE_RIGHT;
      newChain.addEvent(changelaneRight);

      for (int j=eventNo+1; j<oldChain.getEventAmount(); j++)
         newChain.addEvent(oldChain.getEvent(j));
      
      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         // Set the right begin and end coordinates
         Event startEvent = newChain.getEvent(0);
         startEvent.startPosX = -depassStartPoint;
         startEvent.startPosY = 1.5f;
         Event endEvent = newChain.getEvent(eventNo-1);
         endEvent.endPosX = -depassStart;
         endEvent.endPosY = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(newChain,0,eventNo);
         ppBefore.plan();

         // Add three events for the depassing: change_lane left, drive_forward, change_lane right
         Event cll = newChain.getEvent(eventNo);
         cll.startPosX = -depassStart;
         cll.startPosY = 1.5f;
         cll.endPosX = -6.0f;
         cll.endPosY = -1.5f;

         Event df = newChain.getEvent(eventNo+1);
         df.startPosX = -6.0f;
         df.startPosY = -1.5f;
         df.endPosX = 6.0f;
         df.endPosY = -1.5f;

         Event clr = newChain.getEvent(eventNo+2);
         clr.startPosX = 6.0f;
         clr.startPosY = -1.5f;
         clr.endPosX = depassStart;
         clr.endPosY = 1.5f;

         // Set the right begin and end coordinates
         startEvent = newChain.getEvent(eventNo+3);
         startEvent.startPosX = depassStart;
         startEvent.startPosY = 1.5f;
         endEvent = newChain.getEvent(newChain.getEventAmount()-1);
         endEvent.endPosX = depassEndPoint;
         endEvent.endPosY = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter 
            = new PathPlanner(newChain,eventNo+3,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         // Set the right begin and end coordinates
         Event startEvent = newChain.getEvent(0);
         startEvent.startPosX = depassStartPoint;
         startEvent.startPosY = -1.5f;
         Event endEvent = newChain.getEvent(eventNo-1);
         endEvent.endPosX = depassStart;
         endEvent.endPosY = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(newChain,0,eventNo);
         ppBefore.plan();

         // Add three events for the depassing: change_lane left, drive_forward, change_lane right
         Event cll = newChain.getEvent(eventNo);
         cll.startPosX = depassStart;
         cll.startPosY = -1.5f;
         cll.endPosX = 6.0f;
         cll.endPosY = 1.5f;

         Event df = newChain.getEvent(eventNo+1);
         df.startPosX = 6.0f;
         df.startPosY = 1.5f;
         df.endPosX = -6.0f;
         df.endPosY = 1.5f;

         Event clr = newChain.getEvent(eventNo+2);
         clr.startPosX = -6.0f;
         clr.startPosY = 1.5f;
         clr.endPosX = -depassStart;
         clr.endPosY = -1.5f;

         // Set the right begin and end coordinates
         startEvent = newChain.getEvent(eventNo+3);
         startEvent.startPosX = -depassStart;
         startEvent.startPosY = -1.5f;
         endEvent = newChain.getEvent(newChain.getEventAmount()-1);
         endEvent.endPosX = -depassEndPoint;
         endEvent.endPosY = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter 
            = new PathPlanner(newChain,eventNo+3,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         // Set the right begin and end coordinates
         Event startEvent = newChain.getEvent(0);
         startEvent.startPosY = depassStartPoint;
         startEvent.startPosX = 1.5f;
         Event endEvent = newChain.getEvent(eventNo-1);
         endEvent.endPosY = depassStart;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(newChain,0,eventNo);
         ppBefore.plan();

         // Add three events for the depassing: change_lane left, drive_forward, change_lane right
         Event cll = newChain.getEvent(eventNo);
         cll.startPosY = depassStart;
         cll.startPosX = 1.5f;
         cll.endPosY = 6.0f;
         cll.endPosX = -1.5f;

         Event df = newChain.getEvent(eventNo+1);
         df.startPosY = 6.0f;
         df.startPosX = -1.5f;
         df.endPosY = -6.0f;
         df.endPosX = -1.5f;

         Event clr = newChain.getEvent(eventNo+2);
         clr.startPosY = -6.0f;
         clr.startPosX = -1.5f;
         clr.endPosY = -depassStart;
         clr.endPosX = 1.5f;

         // Set the right begin and end coordinates
         startEvent = newChain.getEvent(eventNo+3);
         startEvent.startPosY = -depassStart;
         startEvent.startPosX = 1.5f;
         endEvent = newChain.getEvent(newChain.getEventAmount()-1);
         endEvent.endPosY = -depassEndPoint;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter 
            = new PathPlanner(newChain,eventNo+3,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         // Set the right begin and end coordinates
         Event startEvent = newChain.getEvent(0);
         startEvent.startPosY = -depassStartPoint;
         startEvent.startPosX = -1.5f;
         Event endEvent = newChain.getEvent(eventNo-1);
         endEvent.endPosY = -depassStart;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(newChain,0,eventNo);
         ppBefore.plan();

         // Add three events for the depassing: change_lane left, drive_forward, change_lane right
         Event cll = newChain.getEvent(eventNo);
         cll.startPosY = -depassStart;
         cll.startPosX = -1.5f;
         cll.endPosY = -6.0f;
         cll.endPosX = 1.5f;

         Event df = newChain.getEvent(eventNo+1);
         df.startPosY = -6.0f;
         df.startPosX = 1.5f;
         df.endPosY = 6.0f;
         df.endPosX = 1.5f;

         Event clr = newChain.getEvent(eventNo+2);
         clr.startPosY = 6.0f;
         clr.startPosX = 1.5f;
         clr.endPosY = depassStart;
         clr.endPosX = -1.5f;
         
         // Set the right begin and end coordinates
         startEvent = newChain.getEvent(eventNo+3);
         startEvent.startPosY = depassStart;
         startEvent.startPosX = -1.5f;
         endEvent = newChain.getEvent(newChain.getEventAmount()-1);
         endEvent.endPosY = depassEndPoint;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter 
            = new PathPlanner(newChain,eventNo+3,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      }
      ainfo.getDynamicInfo(index).ceb = newChain;

   }
   
   // This method generates the event chain of SCENARIO_IV
   private void scenarioIV(int index) {
      
      // Count the number of driving_forward & stop units before:
      int eventNo = 0;
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      while (ceb.getEvent(eventNo).kind != Constant.CHANGELANE_LEFT)
            eventNo++;
      
      // Count the number of driving_forward & stop units after:
      int eventNoAfter = ceb.getEventAmount() - eventNo - 1;
      
      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = 1.5f;
         startEvent.startPosX = -depassStartPoint;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = 1.5f;
         endEvent.endPosX = -6.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = -6.0f;
         e.startPosY = 1.5f;
         e.endPosX = 6.0f;
         e.endPosY = -1.5f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = -1.5f;
         startEvent.startPosX = 6.0f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = -1.5f;
         endEvent.endPosX = depassEndPoint;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -1.5f;
         startEvent.startPosX = depassStartPoint;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = -1.5f;
         endEvent.endPosX = 6.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = 6.0f;
         e.startPosY = -1.5f;
         e.endPosX = -6.0f;
         e.endPosY = 1.5f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = 1.5f;
         startEvent.startPosX = -6.0f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = 1.5f;
         endEvent.endPosX = -depassEndPoint;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = depassStartPoint;
         startEvent.startPosX = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = 6.0f;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = 1.5f;
         e.startPosY = 6.0f;
         e.endPosX = -1.5f;
         e.endPosY = -6.0f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = -6.0f;
         startEvent.startPosX = -1.5f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = -depassEndPoint;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -depassStartPoint;
         startEvent.startPosX = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = -6.0f;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = -1.5f;
         e.startPosY = -6.0f;
         e.endPosX = 1.5f;
         e.endPosY = 6.0f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = 6.0f;
         startEvent.startPosX = 1.5f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = depassEndPoint;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      }
   }

   // This method generates the event chain of SCENARIO_V
   private void scenarioV(int index) {
      
      // Count the number of driving_forward & stop units before:
      int eventNo = 0;
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      while (ceb.getEvent(eventNo).kind != Constant.CHANGELANE_RIGHT)
            eventNo++;
      
      // Count the number of driving_forward & stop units after:
      int eventNoAfter = ceb.getEventAmount() - eventNo - 1;
      
      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -1.5f;
         startEvent.startPosX = -depassStartPoint;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = -1.5f;
         endEvent.endPosX = -6.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = -6.0f;
         e.startPosY = -1.5f;
         e.endPosX = 6.0f;
         e.endPosY = 1.5f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = 1.5f;
         startEvent.startPosX = 6.0f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = 1.5f;
         endEvent.endPosX = depassEndPoint;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = 1.5f;
         startEvent.startPosX = depassStartPoint;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = 1.5f;
         endEvent.endPosX = 6.0f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = 6.0f;
         e.startPosY = 1.5f;
         e.endPosX = -6.0f;
         e.endPosY = -1.5f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = -1.5f;
         startEvent.startPosX = -6.0f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = -1.5f;
         endEvent.endPosX = -depassEndPoint;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = depassStartPoint;
         startEvent.startPosX = -1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = 6.0f;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = -1.5f;
         e.startPosY = 6.0f;
         e.endPosX = 1.5f;
         e.endPosY = -6.0f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = -6.0f;
         startEvent.startPosX = 1.5f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = -depassEndPoint;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         // Set the right begin and end coordinates
         Event startEvent = ceb.getEvent(0);
         startEvent.startPosY = -depassStartPoint;
         startEvent.startPosX = 1.5f;
         Event endEvent = ceb.getEvent(eventNo-1);
         endEvent.endPosY = -6.0f;
         endEvent.endPosX = 1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppBefore = new PathPlanner(ceb,0,eventNo);
         ppBefore.plan();

         // Create the changelane event
         Event e = ceb.getEvent(eventNo);
         e.startPosX = 1.5f;
         e.startPosY = -6.0f;
         e.endPosX = -1.5f;
         e.endPosY = 6.0f;

         // Set the right begin and end coordinates
         startEvent = ceb.getEvent(eventNo+1);
         startEvent.startPosY = 6.0f;
         startEvent.startPosX = -1.5f;
         endEvent = ceb.getEvent(ceb.getEventAmount()-1);
         endEvent.endPosY = depassEndPoint;
         endEvent.endPosX = -1.5f;
         
         // Make a pathplanner and start it
         PathPlanner ppAfter = new PathPlanner(ceb,eventNo+1,eventNoAfter);
         ppAfter.plan();

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      }
   }

   // This method generates the event chain of SCENARIO_VI
   public void scenarioVI(int index) { 
      
      EventChain ceb = ainfo.getDynamicInfo(index).ceb;
      
      // If there is only one event 'driving_forward, add another driving forward event, so that the planner
      // generates a vehicle that crosses the crossroad
      if (ceb.getEventAmount() == 1 && ceb.getEvent(0).kind == Constant.STRAIGHTROAD) {
         Event e = new Event();
         e.kind = Constant.STRAIGHTROAD;
         e.name = "event_XXX (generated by planner): "+ainfo.getDynamicObject(index).getId()+" driving forward";
         ceb.addEvent(e);
         drivingforwardCount++;
      }

      EventChain newChain = new EventChain();
      int eventNo=0;
      
      // Copy all stops
      while (eventNo<ceb.getEventAmount() && ceb.getEvent(eventNo).kind == Constant.STOP) {
         Event e = ceb.getEvent(eventNo);
         newChain.addEvent(e);
         eventNo++;
      }
      if (drivingforwardCount > 0) {
         Event e = ceb.getEvent(eventNo);
         newChain.addEvent(e);
         eventNo++;
      }
      // Copy all stops
      while (eventNo<ceb.getEventAmount() && ceb.getEvent(eventNo).kind == Constant.STOP) {
         Event e = ceb.getEvent(eventNo);
         newChain.addEvent(e);
         eventNo++;
      }
      Event criticalEvent = new Event();
      criticalEvent.name = "event_XXX (generated by planner): "+ainfo.getDynamicObject(index).getId()+" driving forward";
      criticalEvent.kind = Constant.STRAIGHTROAD;
      newChain.addEvent(criticalEvent);
      
      // Copy the rest of the events
      int i = eventNo;
      while (i<ceb.getEventAmount()) {
         Event e = ceb.getEvent(i);
         newChain.addEvent(e);
         i++;
      }
      int eventNoAfter = ceb.getEventAmount() - eventNo;

      if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.EAST) {
         if (ceb.getEvent(0).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = -startPoint + 14.0f;
            startEvent.startPosY = 1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = endPoint;
            endEvent.endPosY = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         } else if (ceb.getEventAmount() > 1 
                    && ceb.getEvent(0).kind == Constant.STRAIGHTROAD 
                    && ceb.getEvent(1).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = -startPoint;
            startEvent.startPosY = 1.5f;
            Event endEvent = ceb.getEvent(1);
            endEvent.endPosX = -startPoint + 14.0f;
            endEvent.endPosY = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner ppBefore = new PathPlanner(ceb,0,2);
            ppBefore.plan();
            
            if (ceb.getEventAmount() > 2) {
               startEvent = ceb.getEvent(2);
               startEvent.startPosX = -startPoint + 14.0f;
               startEvent.startPosY = 1.5f;
               endEvent = ceb.getEvent(ceb.getEventAmount()-1);
               endEvent.endPosX = endPoint;
               endEvent.endPosY = 1.5f;
         
               // Make a pathplanner and start it
               PathPlanner ppAfter = new PathPlanner(ceb,2,ceb.getEventAmount()-2);
               ppAfter.plan();
            }
         } else {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = -startPoint;
            startEvent.startPosY = 1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = endPoint;
            endEvent.endPosY = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.WEST) {
         if (ceb.getEvent(0).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = startPoint - 14.0f;
            startEvent.startPosY = -1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = -endPoint;
            endEvent.endPosY = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         } else if (ceb.getEventAmount() > 1 
                    && ceb.getEvent(0).kind == Constant.STRAIGHTROAD 
                    && ceb.getEvent(1).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = startPoint;
            startEvent.startPosY = -1.5f;
            Event endEvent = ceb.getEvent(1);
            endEvent.endPosX = startPoint - 14.0f;
            endEvent.endPosY = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner ppBefore = new PathPlanner(ceb,0,2);
            ppBefore.plan();
            
            if (ceb.getEventAmount() > 2) {
               startEvent = ceb.getEvent(2);
               startEvent.startPosX = startPoint - 14.0f;
               startEvent.startPosY = -1.5f;
               endEvent = ceb.getEvent(ceb.getEventAmount()-1);
               endEvent.endPosX = -endPoint;
               endEvent.endPosY = -1.5f;
         
               // Make a pathplanner and start it
               PathPlanner ppAfter = new PathPlanner(ceb,2,ceb.getEventAmount()-2);
               ppAfter.plan();
            }
         } else {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosX = startPoint;
            startEvent.startPosY = -1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosX = -endPoint;
            endEvent.endPosY = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         }

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 15.0f;
            rsStart.yPos = -3.5f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -15.0f;
            rsEnd.yPos = -3.5f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.NORTH) {
         if (ceb.getEvent(0).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = startPoint - 14.0f;
            startEvent.startPosX = 1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = -endPoint;
            endEvent.endPosX = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         } else if (ceb.getEventAmount() > 1 
                    && ceb.getEvent(0).kind == Constant.STRAIGHTROAD 
                    && ceb.getEvent(1).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = startPoint;
            startEvent.startPosX = 1.5f;
            Event endEvent = ceb.getEvent(1);
            endEvent.endPosY = startPoint - 14.0f;
            endEvent.endPosX = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner ppBefore = new PathPlanner(ceb,0,2);
            ppBefore.plan();
            
            if (ceb.getEventAmount() > 2) {
               startEvent = ceb.getEvent(2);
               startEvent.startPosY = startPoint - 14.0f;
               startEvent.startPosX = 1.5f;
               endEvent = ceb.getEvent(ceb.getEventAmount()-1);
               endEvent.endPosY = -endPoint;
               endEvent.endPosX = 1.5f;
         
               // Make a pathplanner and start it
               PathPlanner ppAfter = new PathPlanner(ceb,2,ceb.getEventAmount()-2);
               ppAfter.plan();
            }
         } else {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = startPoint;
            startEvent.startPosX = 1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = -endPoint;
            endEvent.endPosX = 1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         }
         
         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = -3.5f;
            rsStart.yPos = 15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = 3.5f;
            rsEnd.yPos = -15.0f;
         }
      } else if (ainfo.getDynamicInfo(index).initDirection == DynamicInfo.SOUTH) {
         if (ceb.getEvent(0).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = -startPoint + 14.0f;
            startEvent.startPosX = -1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = endPoint;
            endEvent.endPosX = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         } else if (ceb.getEventAmount() > 1 
                    && ceb.getEvent(0).kind == Constant.STRAIGHTROAD 
                    && ceb.getEvent(1).kind == Constant.STOP) {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = -startPoint;
            startEvent.startPosX = -1.5f;
            Event endEvent = ceb.getEvent(1);
            endEvent.endPosY = -startPoint + 14.0f;
            endEvent.endPosX = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner ppBefore = new PathPlanner(ceb,0,2);
            ppBefore.plan();
            
            if (ceb.getEventAmount() > 2) {
               startEvent = ceb.getEvent(2);
               startEvent.startPosY = -startPoint + 14.0f;
               startEvent.startPosX = -1.5f;
               endEvent = ceb.getEvent(ceb.getEventAmount()-1);
               endEvent.endPosY = endPoint;
               endEvent.endPosX = -1.5f;
         
               // Make a pathplanner and start it
               PathPlanner ppAfter = new PathPlanner(ceb,2,ceb.getEventAmount()-2);
               ppAfter.plan();
            }
         } else {
            Event startEvent = ceb.getEvent(0);
            startEvent.startPosY = -startPoint;
            startEvent.startPosX = -1.5f;
            Event endEvent = ceb.getEvent(ceb.getEventAmount()-1);
            endEvent.endPosY = endPoint;
            endEvent.endPosX = -1.5f;
            
            // Make a pathplanner and start it
            PathPlanner pp = new PathPlanner(ceb,0,ceb.getEventAmount());
            pp.plan();
         }

         // place the roadsigns
         if (ainfo.getDynamicInfo(index).startsignIndex != -1) {
            Roadsign rsStart = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).startsignIndex);
            rsStart.xPos = 3.5f;
            rsStart.yPos = -15.0f;
         }
         if (ainfo.getDynamicInfo(index).endsignIndex != -1) {
            Roadsign rsEnd = (Roadsign)ainfo.getStaticObject(ainfo.getDynamicInfo(index).endsignIndex);
            rsEnd.xPos = -3.5f;
            rsEnd.yPos = 15.0f;
         }
      } 
   }
   
   // Returns true if there are errors detected.
   public boolean errors() {
      return errorsDetected;
   }

}   
