// This planner plans all the collisions that occur in the accident description
// Uses the Circle Cutting Algorithm

// 	Name: MultipleAccidentPlanner (class)
//	Author: Arjan Egges
//	Version: 1.0


import javax.vecmath.*;

class MultipleAccidentPlanner implements Planner {

   // Algorithm properties
   public float initCircleRadius = 3.0f;
   public float radiusStepSize = 0.3f;
   public float maxRadius = 15.0f;
   
   // Collision coordinates
   public float collCoordX = 0.0f;
   public float collCoordY = 0.0f;
   
   // Private members
   private AccidentInfo ainfo; 
   private boolean errorsDetected;  
   private float currRadius;
   private float errorRange = 0.00001f;

   // Constructor
   public MultipleAccidentPlanner(AccidentInfo ai) {
      ainfo = ai;
      errorsDetected = false;
      
      // Set the general planning properties
      initCircleRadius = Messenger.properties.initCircleRadius;
      maxRadius = Messenger.properties.maxSearchRadius;
   }
   
   // Starts the planning process
   public void plan() {
   
      // If there are no collisions defined, no accident modeling is
      // required
      if (ainfo.getCollisionInfoAmount() == 0) {
         Messenger.message("No accident modeling required.");
         return;
      }
      for (int vehIndex=0; vehIndex < ainfo.getDynamicAmount(); vehIndex++) {
         // Get the trajectory of the vehicle
         Trajectory vTraject = ainfo.getDynamicObject(vehIndex).getTrajectory();
         
         // Check if there is a critical region defined.
         int i=0;
         while (i<vTraject.getAmount() && !vTraject.get(i).critical)
            i++;
         // If there is no critical region defined, mark all trajectory segments as critical
         if (i == vTraject.getAmount()) {
            for (int j=0; j<vTraject.getAmount(); j++)
               vTraject.get(j).critical = true;
         }
         
         for (int collIndex=0; collIndex < ainfo.getCollisionInfoAmount(); collIndex++) {
            // Check if the vehicle is actor or victim of the collision
            if (vehIndex == ainfo.getCollisionInfo(collIndex).actorIndex
                || vehIndex == ainfo.getCollisionInfo(collIndex).victimIndex) {
               // Set the collision coordinates
               collCoordX = ainfo.getCollisionInfo(collIndex).collCoordX;
               collCoordY = ainfo.getCollisionInfo(collIndex).collCoordY;
               
               // if it is a static accident, calculate the collision coordinates
               if (ainfo.getCollisionInfo(collIndex).victimKind == Constant.STATIC_OBJECT) {
                  AccidentObject ao = ainfo.getStaticObject(ainfo.getCollisionInfo(collIndex).victimIndex);
                  StaticAccidentCalculator sap = 
                     new StaticAccidentCalculator(ao);
                  // Find the first critical event
                  int critEv = 0;
                  while (critEv < vTraject.getAmount() && !vTraject.get(critEv).critical)
                     critEv++;
                  if (vehIndex == 0) {
                     sap.xPos = vTraject.get(critEv).x;
                     sap.yPos = vTraject.get(critEv).y;
                  } else {
                     sap.xPos = vTraject.get(vTraject.getAmount()).x;
                     sap.yPos = vTraject.get(vTraject.getAmount()).y;
                  }
                  sap.vehiclepart = ainfo.getCollisionInfo(collIndex).actorPart;
                  sap.calculate();
                  collCoordX = sap.endPosX;
                  collCoordY = sap.endPosY;
                  ainfo.getCollisionInfo(collIndex).collCoordX = collCoordX;
                  ainfo.getCollisionInfo(collIndex).collCoordY = collCoordX;

               }
               
               // If the victim is a stopped car, recalculate the collision coordinates
               int collind = collisionInStop(ainfo.getDynamicObject(ainfo.getCollisionInfo(collIndex).victimIndex).getTrajectory()); 
               if (collind != -1) {
                  // Recalculate the collision coordinates
                  
                  // First get the coordinates of the stopped vehicle
                  Trajectory stoppedVehicleTraj = ainfo.getDynamicObject(ainfo.getCollisionInfo(collIndex).victimIndex).getTrajectory();
                  float stopCoordX = stoppedVehicleTraj.get(collind).x;
                  float stopCoordY = stoppedVehicleTraj.get(collind).y;
                  
                  // Calculate the angle of the stopped vehicle
                  float oldX = stoppedVehicleTraj.get(collind-1).x;
                  float oldY = stoppedVehicleTraj.get(collind-1).y;
                  Vector2f a = new Vector2f(stopCoordX-oldX, stopCoordY-oldY);
                  float angle = Trajectory.calcAngle(a);
                  
                  // Get the vehicle's lenght and width
                  float vLength = ainfo.getDynamicObject(ainfo.getCollisionInfo(collIndex).victimIndex).getLength();
                  float vWidth = ainfo.getDynamicObject(ainfo.getCollisionInfo(collIndex).victimIndex).getWidth();
                  
                  // Calculate the (untransformed) new collision coordinates.
                  float newCollCoordX;
                  float newCollCoordY;
                  if ( ainfo.getCollisionInfo(collIndex).victimPart == Constant.FRONT ) {
                     newCollCoordX = vLength/2.0f;
                     newCollCoordY = 0.0f;
                  } else if ( ainfo.getCollisionInfo(collIndex).victimPart == Constant.LEFTSIDE ) {
                     newCollCoordY = vWidth/2.0f;
                     newCollCoordX = 0.0f;
                  } else if ( ainfo.getCollisionInfo(collIndex).victimPart == Constant.REAR ) {
                     newCollCoordX = -vLength/2.0f;
                     newCollCoordY = 0.0f;
                  } else {
                     newCollCoordY = -vWidth/2.0f;
                     newCollCoordX = 0.0f;
                  }
                  // If the direction is north or south, flip x and y values
                  if (ainfo.getDynamicInfo(vehIndex).initDirection == DynamicInfo.NORTH ||
                        ainfo.getDynamicInfo(vehIndex).initDirection == DynamicInfo.SOUTH) {
                     float tmp = -newCollCoordY;
                     newCollCoordY = -newCollCoordX;
                     newCollCoordX = tmp;
                  }
                  
                  // Set the collision coordinates, with rotational and translational corrections
                  collCoordX = newCollCoordX*(float)Math.cos(angle) + stopCoordX;
                  collCoordY = newCollCoordY*(float)Math.sin(angle) + stopCoordY;
                  ainfo.getCollisionInfo(collIndex).collCoordX = collCoordX;
                  ainfo.getCollisionInfo(collIndex).collCoordY = collCoordY;
               
               }

               // Check if it is the first collision for this vehicle
               if (vTraject.get(vTraject.getAmount()-1).collisionIndex == -1) {                  
                  
                  currRadius = initCircleRadius;
                  // Calculate maximum collision radius for the vehicle.
                  float maxRad = 
                     (float)Math.max((double)ainfo.getDynamicObject(vehIndex).getWidth()/2,
                                     (double)ainfo.getDynamicObject(vehIndex).getLength()/2);
                  if (initCircleRadius < maxRad)
                     initCircleRadius = maxRad;
                  // Call the circle algorithm
                  cutTrajectory(vTraject);
               }
               
               // Plan the collision
               Event vEvent = new Event();
               vEvent.startPosX = vTraject.get(vTraject.getAmount()-1).x;
               vEvent.startPosY = vTraject.get(vTraject.getAmount()-1).y;
               CollisionPlanner vPlanner = new CollisionPlanner(vEvent);
               vPlanner.collX = collCoordX;
               vPlanner.collY = collCoordY;
               if (vehIndex == ainfo.getCollisionInfo(collIndex).actorIndex)
                  vPlanner.vehiclepart = ainfo.getCollisionInfo(collIndex).actorPart;
               else
                  vPlanner.vehiclepart = ainfo.getCollisionInfo(collIndex).victimPart;
               
               vPlanner.plan();
      
               if (!(vehIndex == ainfo.getCollisionInfo(collIndex).victimIndex 
                     && 
                     collisionInStop(ainfo.getDynamicObject(vehIndex).getTrajectory()) != -1
                   )) {
                  // Add the collision point to the trajectory and set the collision index
                  vTraject.add(new TrajectoryPoint(vEvent.endPosX,vEvent.endPosY,0.0f));
                  vTraject.get(vTraject.getAmount()-1).collisionIndex = collIndex;
               }
            }
         }
      }
   }
   
   // This method cuts the trajectory using the circle algorithm
   // If the collision occurs within the stop, all the segments after the stop are removed.
   private void cutTrajectory(Trajectory t) {
      if (currRadius > maxRadius) {
         Messenger.message("Unable to plan accident. Collision point out of bounds.\n");
         errorsDetected = true;
         return;
      }
      // Remove all the non-significant trajectory segments
      int currTrajIndex = 0;
      TrajectoryPoint point1 = new TrajectoryPoint();
      TrajectoryPoint point2 = new TrajectoryPoint();


      // If the car has a collision while it has stopped, just cut all trajectorysegments after the stop
      int collind = collisionInStop(t);
      if (collind != -1) {
         collind += 2;
         while (t.getAmount() > collind)
            t.delete();  
         return;
      }

      while (currTrajIndex < t.getAmount()) {
         if (currTrajIndex == 0) {
            point2 = t.get(currTrajIndex);
         } else {
            point1 = new TrajectoryPoint(point2);
            point2 = t.get(currTrajIndex);
            
            // If the trajectorysegment is a stop and it is marked critical, do not
            // call the circle algorithm.
            
            // Check if the line lies within the circle if it is critical.
            if (isCritical(point1,point2) &&
                isInCircleRegion(point1,point2,
                                 collCoordX,
                                 collCoordY,
                                 currRadius)) {
               // Throw away all segments after the currTrajIndex (incl.)
               while (currTrajIndex < t.getAmount())
                  t.delete();
               return;
            }
         }
         currTrajIndex++;
      } 
      // Here the trajectory was not cut.
      currRadius += radiusStepSize;
      cutTrajectory(t);     
   }
   
   // Returns true if the segment p1 - p2 is critical.
   private boolean isCritical(TrajectoryPoint p1, TrajectoryPoint p2) {
      return (p1.critical && p2.critical);
   }
   
   // Checks if the stop is defined as 'critical', if so, this method returns the index of the first trajectorypoint
   // of the stop, else it returns -1.
   private int collisionInStop(Trajectory t) {
      int currInd = 1;
      TrajectoryPoint point1;
      TrajectoryPoint point2;
      boolean found = false;
      while (currInd<t.getAmount() && !found) {
         point1 = t.get(currInd-1);
         point2 = t.get(currInd);
         if (isCritical(point1,point2) && isStop(point1,point2)) 
            found = true;
         currInd++;
      }
      if (!found)
         return -1;
      else
         return (currInd-2);
   }

   // Checks if two trajectorypoints form a stop
   private boolean isStop(TrajectoryPoint p1, TrajectoryPoint p2) {
      float deltaX = (float)Math.abs((float)(p1.x - p2.x));
      float deltaY = (float)Math.abs((float)(p1.y - p2.y));
      
      return ((deltaX < errorRange) && (deltaY < errorRange));
   }
   
   // Returns true if the line segment p1 - p2 lies (partially) in the circle (cx, cy) with 
   // radius cRadius 
   private boolean isInCircleRegion(TrajectoryPoint p1,
                                    TrajectoryPoint p2,
                                    float cx,
                                    float cy,
                                    float cRadius) {
      double dx = (double)(p2.x - p1.x);
      double dy = (double)(p2.y - p1.y);
      double dr2 = Math.pow(dx,2) + Math.pow(dy,2);
      
      double D = (double)((p1.x-cx)*(p2.y-cy) - (p2.x-cx)*(p1.y-cy));
      double discriminant = Math.pow((double)cRadius,2)*dr2 - Math.pow(D,2);
      
      if (discriminant < 0)	// No intersection at all
         return false;
      
      // The line y = ax+b defined by the points p1 and p2 passes through the circle.
      
      // Calculate the intersection points
      TrajectoryPoint int1 = new TrajectoryPoint();
      TrajectoryPoint int2 = new TrajectoryPoint();
      int1.x = (float)((D*dy + sgn(dy)*dx*Math.sqrt(discriminant))/dr2);
      int1.y = (float)((-D*dx + Math.abs(dy)*Math.sqrt(discriminant))/dr2);
      int2.x = (float)((D*dy - sgn(dy)*dx*Math.sqrt(discriminant))/dr2);
      int2.y = (float)((-D*dx - Math.abs(dy)*Math.sqrt(discriminant))/dr2);
      
      if (liesInCircle(p1,cx,cy,cRadius) || liesInCircle(p2,cx,cy,cRadius))
         return true;
      else {
         // Check if one of the intersectionpoints lies on the line segment
         return (liesInRectangle(p1,p2,int1) || liesInRectangle(p1,p2,int2));
      }
   }
   
   private double sgn(double d) {
      if (d<0)
         return -1;
      else
         return 1;         
   }
   
   // Returns true if the point p lies in the circle with center(cx,cy) and radius cRadius   
   private boolean liesInCircle(TrajectoryPoint p, float cx, float cy, float cRadius) {
      double r2 = Math.pow((double)(p.x-cx),2) + Math.pow((double)(p.y-cy),2);
      return (r2 <= Math.pow(cRadius,2));
   }
   
   // Returns true if the point tp lies in or on the rectangle defined by sp1 and sp2
   private boolean liesInRectangle(TrajectoryPoint sp1, TrajectoryPoint sp2, TrajectoryPoint tp) {
      double minX = Math.min((double)sp1.x,(double)sp2.x);
      double minY = Math.min((double)sp1.y,(double)sp2.y);
      double maxX = Math.max((double)sp1.x,(double)sp2.x);
      double maxY = Math.max((double)sp1.y,(double)sp2.y);
      return (tp.x >= minX && tp.x <= maxX && tp.y >= minY && tp.y <= maxY);
   }
   
   // Returns true if there are errors detected
   public boolean errors() {
      return errorsDetected;
   }
}