// This class plans a trajectory of stops and driving-forward's between to given points p1, p2
// p1 is defined by the startpos of the first event to be changed
// p2 is defined by the endpos of the last event to be changed.

// 	Name: PathPlanner (class)
//	Author: Arjan Egges
//	Version: 1.0

class PathPlanner implements Planner {

   // Private members
   private EventChain eventChain;
   private int startIndex;
   private int numberOfEvents;
   private boolean errorsDetected = false;
   
   private float beginX = 0.0f;
   private float beginY = 0.0f;
   private float endX = 0.0f;
   private float endY = 0.0f;
   
   // Constructor
   public PathPlanner(EventChain ceb, 			// The event chain to be modified
   		      int si,				// The index from with to start modifying
   		      int eventNo) {			// The number of events to modify
   		      
   		      
      eventChain = ceb;
      startIndex = si;
      numberOfEvents = eventNo;
   }
   
   // Starts the planning process
   public void plan() {
      // If one of the events is not driving_forward or stop, an error is detected.
      // If the eventAmount exceeds the eventchain size, an error is detected.
      if (startIndex+numberOfEvents > eventChain.getEventAmount()) {
         errorsDetected = true;
         return;
      }
      if (numberOfEvents == 0)
         return;
         
      beginX = eventChain.getEvent(startIndex).startPosX;
      beginY = eventChain.getEvent(startIndex).startPosY;
      endX = eventChain.getEvent(startIndex+numberOfEvents-1).endPosX;
      endY = eventChain.getEvent(startIndex+numberOfEvents-1).endPosY;
      
      // At this point the planning can begin
      planPath();
   }
   
   // This method actually plans the path
   private void planPath() {
      // Count the number of driving_forward & stop units:
      int drForwCount = 0;
      int stopCount = 0;
      for (int i = startIndex; i< startIndex+numberOfEvents; i++) {
         if (eventChain.getEvent(i).kind == Constant.STRAIGHTROAD)
            drForwCount++;
         else if (eventChain.getEvent(i).kind == Constant.STOP)
            stopCount++;
         else {
            errorsDetected = true;
            return;
         }
      }
      
      // Calculate the values for x & y incrementing
      float xincr = (endX - beginX)/(float)drForwCount;
      float yincr = (endY - beginY)/(float)drForwCount;
      
      float startX = beginX;
      float startY = beginY;
      
      for (int i = startIndex; i< startIndex+numberOfEvents; i++) {
         Event e = eventChain.getEvent(i);
         e.startPosX = startX;
         e.startPosY = startY;
         
         if (e.kind == Constant.STRAIGHTROAD) {
            startX += xincr;
            startY += yincr;
            e.endPosX = startX;
            e.endPosY = startY;
         
         } else {
            e.endPosX = startX;
            e.endPosY = startY;
         }
      }    
   }
   
   // Returns true if there are errors detected
   public boolean errors() {
      return errorsDetected;
   }
   
}