// This creates a standard SimpleCar object.

// 	Name: SimpleCar (class)
//	Author: Arjan Egges
//	Version: 1.0


import com.sun.j3d.utils.behaviors.interpolators.*;
import java.util.Date;
import com.sun.j3d.utils.geometry.*;
import javax.media.j3d.*;
import javax.vecmath.*;

class SimpleCar implements DynamicAccidentObject {

   // Property variables
   public float length;
   public float width;
   public float height;
   public float wheelRadius;
   public float wheelWidth;
   public String id;

   // Private members
   private TransformGroup movingCar;
   private RotPosScaleTCBSplinePathInterpolator rotPosInt;
   private Trajectory traject;
   private Alpha alpha;
   private long timeStamp;
   private long startTime;  

   // Constructor
   public SimpleCar() {
      length = 3.5f;
      width = 1.5f;
      height = 1.0f;
      wheelRadius = 0.3f;
      wheelWidth = 0.25f;
      id = "";
      
      // Dynamic properties
      traject = new Trajectory();
      timeStamp = new Date().getTime(); 
   }
 
   // Creates the geometry of the simple car  
   public void createGeometry() {
      // Dynamic property variables
      float startPosX = traject.get(0).x;
      float startPosY = traject.get(0).y;
      
      Transform3D translate = new Transform3D();
      translate.set(new Vector3f(0.0f,0.0f,0.0f));
      TransformGroup carGeo = new TransformGroup(translate);
      
      // A simple car consists of a box and four cylinders
      Appearance carAppear = new Appearance();
      Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      Color3f oColor    = new Color3f(0.9f, 0.4f, 0.6f);
      
      Material carMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      carMat.setLightingEnable(true);
      carAppear.setMaterial(carMat);
            
      Appearance wheelAppear = new Appearance();
      eColor    = new Color3f(0.0f, 0.0f, 0.0f);
      sColor    = new Color3f(1.0f, 1.0f, 1.0f);
      oColor    = new Color3f(0.5f, 0.5f, 0.5f);
      
      Material wheelMat = new Material(oColor, eColor, oColor, sColor, 100.0f);
      wheelMat.setLightingEnable(true);
      wheelAppear.setMaterial(wheelMat);
      
      // Add four cylinders and translate them
      Transform3D rotate = new Transform3D();
      rotate.rotX(Math.PI/2.0f);
      translate.set(new Vector3f((length/2.0f)*0.9f-wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.05f));
      translate.mul(rotate);
      TransformGroup wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      translate.set(new Vector3f((length/2.0f)*0.9f-wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.05f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      translate.set(new Vector3f(-(length/2.0f)*0.9f+wheelRadius, wheelRadius, width/2.0f-wheelWidth-0.05f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      translate.set(new Vector3f(-(length/2.0f)*0.9f+wheelRadius, wheelRadius, -width/2.0f+wheelWidth+0.05f));
      translate.mul(rotate);
      wheelTrans = new TransformGroup(translate);
      wheelTrans.addChild(new Cylinder(wheelRadius, wheelWidth,wheelAppear));
      carGeo.addChild(wheelTrans);
      
      // Commented code is for defining an additional cube to distinguish the
      // front of the vehicle from the back
      
      // translate.set(new Vector3f(2.0f, 1.0f, 1.0f));
      // TransformGroup fr = new TransformGroup(translate);
      // fr.addChild(new Box(0.4f, 0.4f, 0.4f, Box.GENERATE_NORMALS,carAppear));
      // carGeo.addChild(fr);
      
      Box mainCar = new Box(length/2.0f,height/2.0f,width/2.0f,
                           Box.GENERATE_NORMALS,carAppear);
      translate = new Transform3D();
      translate.set(new Vector3f(0.0f, (height/2.0f)+wheelRadius*1.5f, 0.0f));
      TransformGroup carTrans = new TransformGroup(translate);
      carTrans.addChild(mainCar);
      carGeo.addChild(carTrans);
      
      // The next part of the code specifies the behavior of the car. This means traveling along the
      // road between two given points in the XZ plane.
           
      translate.set(new Vector3f(startPosX, 0.0f, startPosY));
      rotate.rotY(traject.getInitAngle());
      translate.mul(rotate);
      movingCar = new TransformGroup(translate);
      
      movingCar.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      
      // Create behavior
      TrajectoryBehavior tb = new TrajectoryBehavior(this);
      tb.createBehavior();
      alpha = tb.alpha;
      
      rotPosInt = tb.getBehavior();
      rotPosInt.setEnable(false);
      movingCar.addChild(carGeo);
      movingCar.addChild(rotPosInt);
   
   }
   
   // Returns the geometry of the simple car
   public TransformGroup getGeometry() {
      return movingCar;
   }
   
   // Sets the trajectory of the SimpleCar object
   public void setTrajectory(Trajectory t) {
      traject = t;
   }
   
   // Returns the trajectory of the SimpleCar object
   public Trajectory getTrajectory() {
      return traject;
   }
   
   // Some methods for handling dynamics 
   
   // Start animation
   public void play() {
      startTime = new Date().getTime()+500;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);    
   }
   
   // Pause animation
   public void pause() {
      timeStamp = new Date().getTime() - startTime;
      rotPosInt.setEnable(false);
   }
   
   // Resume animation
   public void resume() {
      startTime = new Date().getTime() - timeStamp;
      alpha.setStartTime(startTime);
      rotPosInt.setEnable(true);
   }
   
   // Stop animation
   public void stop() {
      rotPosInt.setEnable(false);
   }  
   
   // Returns the id
   public String getId() {
      return id;
   }
   
   // Set the id to s
   public void setId(String s) {
      id = s;
   }
   
   // Returns the length of the vehicle
   public float getLength() {
      return length;
   }
   
   // Returns the width of the vehicle
   public float getWidth() {
      return width;
   }
   

   
}