// This planner plan the temporal values of the trajectory

// 	Name: TemporalPlanner (class)
//	Author: Arjan Egges
//	Version: 1.0


class TemporalPlanner implements Planner {

   // Planning properties
   public float stopPerc = 15.0f;
   public float accPerc = 20.0f;

   // Private members
   private AccidentInfo ainfo;
   private boolean errorsDetected;
   private float errorRange = 0.0001f;
   
   // Synchronisation point array
   private float[] spArr;
   
   // Constructor
   public TemporalPlanner(AccidentInfo ai) {
      ainfo = ai;
      errorsDetected = false;
      
      // Set the general planning properties
      stopPerc = Messenger.properties.stopPerc;
      
      if (ainfo.getCollisionInfoAmount() > 0) {
         // Initialise the synchronisation point array
         spArr = new float[ainfo.getCollisionInfoAmount()];
         // Calculate the synchronisation values
         spArr[0] = 1.0f - accPerc/100.0f;
         int i = 1;
         while (i < ainfo.getCollisionInfoAmount()) {
            spArr[i] = spArr[i-1] + accPerc/100.0f/(ainfo.getCollisionInfoAmount()-1);
            i++;
         }
      } else {
         spArr = new float[1];
         spArr[0] = 1.0f - accPerc/100.0f;
      }
      
   }
   
   // Planning method
   public void plan() {
      // The planning consists of calculating the percentages of every trajectory segment
      for (int index=0; index < ainfo.getDynamicAmount(); index++) 
         calculateTimeStamps(index);
   }
   
   // Calculates the time stamps (percentages) of a trajectory t.
   private void calculateTimeStamps(int i) {
      Trajectory t = ainfo.getDynamicObject(i).getTrajectory();
      if (t.getAmount() == 0)
         return;
      
      int stopCount = 0;
      int currTrajIndex = 0;
            
      // Calculate total distance before the collisions occur
      float totalDistance = 0.0f;
      for (int index=1; index < t.getAmount(); index++) {
         totalDistance += distance(t.get(index-1), t.get(index));
         currTrajIndex = index;
         if (t.get(index).collisionIndex != -1) {
            break;
         }
         if (distance(t.get(index-1), t.get(index)) == 0.0f)
            stopCount++;
      }
      
      float currDistance = 0.0f;
      float currPerc = 0.0f;
      
      // Set the percentage factors of the stops and the other trajectories
      float stopFactor = stopPerc/100.0f;
      float restFactor = spArr[0] - stopFactor;
      if (stopCount == 0)
         restFactor = spArr[0];
      
      // Calculate the new percentages
      t.get(0).percentage = 0.0f;
      for (int index=1; index < t.getAmount(); index++) { 
         float dist = distance(t.get(index-1), t.get(index));
         if (dist == 0.0f)
            currPerc += stopFactor/(float)stopCount;
         else
            currPerc += dist / totalDistance * restFactor;
         t.get(index).percentage = currPerc;
      }  
      
      // Calculate the rest of the collision percentages for this vehicle
      // For every collision, add the synchronisation point to the trajectory.
      for (int index = currTrajIndex; index < t.getAmount(); index++) {
         if (t.get(index).collisionIndex != -1)
            t.get(index).percentage = spArr[t.get(index).collisionIndex];
         else
            t.get(index).percentage = spArr[0];
      }
      
      // If the total percentage != 1.0, add a stop
      if (Math.abs(t.get(t.getAmount()-1).percentage - 1.0f) >= errorRange) {
         TrajectoryPoint tp = new TrajectoryPoint(t.get(t.getAmount()-1));
         tp.percentage = 1.0f;
         t.add(tp);
      }   
   }
   
   // Calculates the distance between two trajectory points p1, p2
   private float distance(TrajectoryPoint p1, TrajectoryPoint p2) {
      double deltaX = (double)(p2.x - p1.x);
      double deltaY = (double)(p2.y - p1.y);
      float d = (float)Math.sqrt(Math.pow(deltaX,2) + Math.pow(deltaY,2));
      
      return d;
   }
   
   // Returns true if there are errors detected
   public boolean errors() {
      return errorsDetected;
   }
}